--------------------------------------------------------------------------------
-- Demo Derby Vehicle Randomiser
-- By Dagalufh & Scepheo
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Scenario config
--------------------------------------------------------------------------------

local vehicleNames = 
{
	"scenario_player0",
	"opponent1",
	"opponent2",
	"opponent3",
}

--------------------------------------------------------------------------------
-- Scenario code
--------------------------------------------------------------------------------

local M = {}

local function getRandomKey(items)
	-- Count the number of items in the table
	local itemCount = 0
	
	for key, value in pairs(items) do
		itemCount = itemCount + 1
	end
	
	-- Select a random number n
	local selectedIndex = math.random(1, itemCount)
	
	-- Get the nth node
	local index = 1
	local selectedKey = ""
	
	for key, value in pairs(items) do
		if index == selectedIndex then
			selectedKey = key
			break
		end
		
		index = index + 1
	end
	
	return selectedKey
end

local function getRandomValue(items)
	return items[getRandomKey(items)]
end

local function onPreStart(scenario)

	-- Get the list of vehicles
	local configList = vehicles.getConfigList().configs
	
	-- Get a new random seed
	math.randomseed(os.time())
	
	-- Replace each vehicle with a random one
	for i, carName in pairs(vehicleNames) do
		local config
		local valid = false
		
		while not valid do
			config = getRandomValue(configList)
			local isCar = config.aggregates.Type.Car
			local isTruck = config.aggregates.Type.Truck
			valid = isCar or isTruck
		end
		
		local jbeam = config.model_key
		local configuration = string.format('vehicles/%s/%s.pc', jbeam, config.key)
		
		scenario.demoNames[i - 1] = config.Name
		
		TorqueScript.eval(
		[[
			]] .. carName .. [[.JBeam = "]] .. jbeam .. [[";
			]] .. carName .. [[.partConfig = "]] .. configuration .. [[";
			]] .. carName .. [[.color = "]] .. config.default_color .. [[";
			]] .. carName .. [[.requestReload();
		]])
	end
end

local function onScenarioChange(scenario)
	if scenario and scenario.state == 'pre-start' then
		onPreStart(scenario)
	end
end

local function onPhysicsEngineEvent(eventType)
	-- Freeze whenever a vehicle is spawned or despawned
	scenetree.scenario_player0:queueLuaCommand('scenario.freeze(1)')
end

M.onScenarioChange = onScenarioChange
M.onPhysicsEngineEvent = onPhysicsEngineEvent

return M